/*
 * deviced
 *
 * Copyright (c) 2012 - 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <ITapiModem.h>
#include <TelPower.h>
#include <tapi_event.h>
#include <tapi_common.h>
#include <glib.h>
#include <unistd.h>
#include "core/log.h"
#include "core/common.h"
#include "core/devices.h"
#include "core/device-notifier.h"
#include "core/edbus-handler.h"
#include "power/power-handler.h"

#define TELEPHONY_NOTI_TIMEOUT 15000 /* milliseconds */

static TapiHandle *tapi_handle;
static enum poweroff_type type;

static void telephony_off(void)
{
	static int already = 0;

	if (already)
		return;

	already = 1;

	switch (type) {
	case POWER_OFF_RESTART:
		restart_ap(NULL);
		break;
	case POWER_OFF_DIRECT:
	default:
		powerdown_ap(NULL);
		break;
	}
}

static void telephony_noti_cb(TapiHandle *handle, const char *noti_id, void *data, void *user_data)
{
	int ret;

	_I("Modem Power Off");

	if (handle) {
		ret = tel_deregister_noti_event(handle, TAPI_NOTI_MODEM_POWER);
		if (ret != TAPI_API_SUCCESS)
			_E("Failed to unregister tapi noti event (%d)", ret);

		ret = tel_deinit(tapi_handle);
		if (ret != TAPI_API_SUCCESS)
			_E("Failed to deinit tapi handle (%d)", ret);
	}

	telephony_off();
}

static void telephony_resp_cb(TapiHandle *handle, int result, void *data, void *user_data)
{
	_D("poweroff command request : %d", result);
}

static gboolean telephony_request_timeout(gpointer data)
{
	_E("No response from Telephony");
	telephony_off();
	return G_SOURCE_REMOVE;
}

static int telephony_poweroff(void *data)
{
	int ret;
	guint id;

	type = *(enum poweroff_type *)data;

	if (!tapi_handle) {
		tapi_handle = tel_init(NULL);
		if (!tapi_handle) {
			_E("Failed to init telephony");
			return -ENOMEM;
		}
	}

	ret = tel_register_noti_event(tapi_handle,
			TAPI_NOTI_MODEM_POWER, telephony_noti_cb, NULL);
	if (ret != TAPI_API_SUCCESS) {
		_E("Failed to register tapi noti event (%d)", ret);
		return (ret < 0 ? ret : -ret);
	}

	ret = tel_process_power_command(tapi_handle,
			TAPI_PHONE_POWER_OFF, telephony_resp_cb, NULL);
	if (ret != TAPI_API_SUCCESS) {
		_E("Failed to notify tapi power command response(%d)", ret);
		return (ret < 0 ? ret : -ret);
	}

	id = g_timeout_add(TELEPHONY_NOTI_TIMEOUT, telephony_request_timeout, NULL);
	if (id == 0) {
		_E("Failed to add timer for force poweroff");
		return -ENOMEM;
	}

	return 0;
}

static int telephony_execute(void *data)
{
	return telephony_poweroff(data);
}

static const struct device_ops tel_device_ops = {
	.name     = "telephony",
	.execute  = telephony_execute,
};

DEVICE_OPS_REGISTER(&tel_device_ops)
